#!/bin/bash

# JJ 21.3.2013: this scripts is called from pam session authentication module
# to create a symlink from /home/luser -> /home/ad/lusers-home-directory/
# and other relevant staff, which cannot be done with plain user access

# mjr 18.4.2013. Handle prefixed profiles.
# mjr 21.5.2014. Ditch profiles for 14.04
# JJ 18.9.2014. handle luser@cs
# JJ 8.3.2016. Remove tktl-csfs

log() {
   logger -t "$0" "[$$] $*"
}

log "Starting: PAM_SERVICE=$PAM_SERVICE PAM_USER=$PAM_USER PAM_TYPE=$PAM_TYPE"

if [ "$PAM_TYPE" != "open_session" ]; then
  log "exiting, PAM_TYPE != open_session"
  exit 0
fi


# Check that we have a uid we like
uid="$(id -u "$PAM_USER")"
if [ "x$uid" == "x" ] || [ $uid -lt 1000 ]; then
  log "Ignoring uid=$uid";
  exit 0
fi

  
# Check login from a CS Department user and try to force right uid to sssd
# cache
# FIXME: this needs to die
luser="$(echo $PAM_USER|cut -d @ -f 1-1)"
if [ "$luser" = "$PAM_USER" ]; then
  # Drop CS user from cache
  /usr/sbin/sss_cache -u "$luser@cs"
else
  # Drop AD user from cache
  /usr/sbin/sss_cache -u "$luser"
fi
getent="$(getent passwd "$PAM_USER")"
# Now check if we need to do cs user stuff
if getent passwd "$luser"@cs > /dev/null; then
  # laptop-detect needs sbin PATHs
  export PATH=/sbin:/usr/sbin:$PATH
  if ! laptop-detect; then
    for d in home group work; do
       mount /cs/$d
    done
  fi
else
  log "Not a CS Department user: $PAM_USER"
fi

if ! getent passwd $PAM_USER | egrep -q ":/home/$PAM_USER:[^:]+$"; then
  log "Exiting for $PAM_USER: home directory not /home/luser, not updating symlinks"
  exit 0
fi

# JJ: Create the user's own group, if it is missing. This attempts to resolve 
# conflict between AD and physics department and ldap-internal
if [ "$uid" -gt 1000000 ] &&  [ -z "$(getent group $uid|fgrep :$PAM_USER)" ]; then
  log "$PAM_USER is missing group entry for $uid. Fixing this."
  groupadd -g "$uid" -p "*added-by-$0*" "$PAM_USER"
  gpasswd --add $PAM_USER $PAM_USER > /dev/null
  nscd -i group
fi

c=$(echo "$PAM_USER" | cut -c1)

case "$c" in
[a-e])
  fshome=1
  ;;
[f-j])
  fshome=2
  ;;
[k-l])
  fshome=3
  ;;
[m-o])
  fshome=4
  ;;
[p-s])
  fshome=5
  ;;
[t-z])
  fshome=6
  ;;
*)
  log "Can't handle user $PAM_USER: invalid starting character"
  exit 1
esac

if [ -e /etc/cubbli/flags/use-ad-home ] ; then
  # Create a /home -> adhome symlink
  adpath=ad/fshome$fshome/u$fshome/$c/$PAM_USER/Linux
  if sudo -n -u $PAM_USER mkdir -p /home/$adpath; then
     if [ "$(readlink /home/$PAM_USER)" != "./$adpath" ]; then
       if ln -T -sf "./$adpath" "/home/$PAM_USER"; then
         log "Symlink /home/$PAM_USER -> ./$adpath created."
       else
         log "Symlink /home/$PAM_USER -> ./$adpath FAILED."
         echo "Failed to create /home/$PAM_USER symlink. Home directory probably is not available." 1>&2       
         exit 1
       fi
     else
       log "Symlink exists."
     fi
  else
    log "Could not sudo to user account."
    echo "Failed to create /home/$PAM_USER symlink. Home directory probably is not available." 1>&2
    exit 1
  fi
  #local_dir="/home/local/$PAM_USER"
  #mkdir -p  --mode 0755 /home/local
else
  # Make workstation user lpadmin
  adduser $PAM_USER lpadmin
  # Remove an adhome symlink if one exists
  local_dir="/home/$PAM_USER"
  [ -L "$local_dir" ] && rm -f "$local_dir"
fi

# Create the local home directory if needed
if [ ! -z "$local_dir" ] && [ ! -d "$local_dir" ]; then
  log "Creating local home directory $local_dir."
  mkdir -p  --mode 0700 "$local_dir"
  chown $(id -u $PAM_USER).$(id -g $PAM_USER) $local_dir
fi

exit 0

